#ifndef STACK_HPP_INCLUDED
#define STACK_HPP_INCLUDED

template <class T> // elemtípus sablonja
class Stack // verem típus
{
    private:
        struct StackItem{ // veremelem típusa, mint beágyazott típu
            T _value; // a tárolt érték
            StackItem* next; // a következő elem mutatója
        };

        StackItem* _top; // a tetőelem mutatója
        int _size; // méret (csak a lekérdezéshez kell)

    public:
        enum Exceptions { STACK_EMPTY }; // kivételek felsorolási típusa

        Stack(); // konstruktor (már nem adunk meg méretet)
        Stack(const Stack& other);
        ~Stack(); // destruktor
        bool isEmpty() const; // üres-e a verem
        void push(T value); // elem behelyezése
        T pop(); // elem kivétele
        T top() const; // tetőelem lekérdezése
        void clear(); // elemek törlése
        int size() const; // méret lekérdezése

        Stack& operator=(const Stack& other);
};

template <class T>
Stack<T>::Stack()
{
    _top = 0; // kezdetben nincs egy elem sem
    _size = 0;
}

template <class T>
Stack<T>::Stack(const Stack<T>& other)
{
    _top = 0;
    _size = other._size;
    StackItem* thisPointer = 0;
    for (StackItem* otherPointer = other._top; otherPointer != 0; otherPointer = otherPointer->next)
    // a ciklust a paraméterben kapott verem láncára írjuk meg
    {
        StackItem* nextItem = new StackItem;
        nextItem->_value = otherPointer->_value;// létrehozunk egy új elemet a memóriában, amely megkapja a másik veremből az adatrészt

        if (_top == 0) // amikor az első elemet helyezzük bele
        {
            _top = nextItem;  // akkor a top mutatót kell állítani
            thisPointer = _top;
        }
        else
        {
            thisPointer->next = nextItem; // különben a thisPointer mutatót
            thisPointer = thisPointer->next; // amellyel folyamatosan haladunk előre a láncban
        }
    }
}

template <class T>
Stack<T>::~Stack()
{
    clear();
}

template <class T>
bool Stack<T>::isEmpty() const
{
    return _size == 0;
}

template <class T>
void Stack<T>::push(T value)
{
    StackItem* item = new StackItem; // létrehozzuk az új elemet
    item->_value = value; // beállítjuk az értékét
    item->next = _top; // ráhelyezzük a lánc eddigi részét
    _top = item; // berakjuk az elejére
    _size++;
}

template <class T>
T Stack<T>::pop()
{
    if (_size > 0) // ha van elem
    {
        T data = _top->_value; // lekérdezzük az elemet
        StackItem* temp = _top; // segédmutató (a törléshez)
        _top = _top->next; // átállítjuk az első elemet
        delete temp; // kitöröljük az elemet
        _size--;

        return data; // visszaadjuk az értéket
    }
    else // különben kivételt dobunk
        throw STACK_EMPTY;
}

template <class T>
T Stack<T>::top() const
{
    if (_size > 0) // ha van elem
        return _top->_value;
    else // különben kivételt dobunk
        throw STACK_EMPTY;
}

template <class T>
void Stack<T>::clear()
{
    while (_size > 0) // egyenként takarítjuk ki az elemeket
        pop();
}

template <class T>
int Stack<T>::size() const
{
    return _size;
}

template <class T>
Stack<T>& Stack<T>::operator=(const Stack<T>& other)
{
    if (this == &other)
        return *this;

    clear();

    _top = 0;
    _size = other._size;
    StackItem* thisPointer = 0;
    for (StackItem* otherPointer = other._top; otherPointer != 0; otherPointer = otherPointer->next)
    // a ciklust a paraméterben kapott verem láncára írjuk meg
    {
        StackItem* nextItem = new StackItem;
        nextItem->_value = otherPointer->_value;// létrehozunk egy új elemet a memóriában, amely megkapja a másik veremből az adatrészt

        if (_top == 0) // amikor az első elemet helyezzük bele
        {
            _top = nextItem;  // akkor a top mutatót kell állítani
            thisPointer = _top;
        }
        else
        {
            thisPointer->next = nextItem; // különben a thisPointer mutatót
            thisPointer = thisPointer->next; // amellyel folyamatosan haladunk előre a láncban
        }
    }

    return *this;
}

#endif // STACK_HPP_INCLUDED
